# CLOSE BLUE IRIS APPLICATION (NOT THE SERVICE) IF PC IS INACTIVE (IDLE) FOR A PERIOD OF TIME;
# AND OPTIONALLY POST MESSAGE TO THE BLUE IRIS LOGFILE AND/OR SEND PUSHOVER NOTIFICATION
#
# Version 3
# Created by jaydeel at ipcamtalk.com
#
# USAGE:      BI_app_stop_if_idle.ps1 [arg]
# ARGUMENTS:  -Demo    = use to simulate how the script works
#             -NoBiLog = use to temporarily override user setting $post_to_BI_logfile_flg
#             -NoPushover = use to temporarily override user setting $send_po_notification_flg
#             -Help    = show syntax help only
# RETURNS:    (string) "Nothing to do..." if no action taken by script
#
# Updates:
# V3 - 2024-01-26 - BREAKING CHANGE: moved USER SETTINGS code section to separate script file 
#                   '{scriptname}_user_settings.ps1' 
#                 - NEW: Added support for killing additional processes (see user_settings file) 
#                 - NEW: Added arguments -Demo, -NoBiLog, -NoPushover, -Help
#                 - CHANGE: Consolidated duplicative code for closing app(s) to new function 
# V2 - 2023-02-13 - IMPROVED: Tweaked handling of response from 'tasklist' command.
#                 - IMPROVED: layout of USER SETTINGS section.
#                 - CHANGE: Migrated code for logging & notifications to new functions 
#                 - (posted at http://tinyurl.com/4upa3fpp)
# V1 - 2023-01-11 - Initial release (http://tinyurl.com/3rpcmmdr)
#
# BEFORE USING THIS SCRIPT:
#    EDIT ALL [REQUIRED] VARIABLES IN THE COMPANION 'USER_SETTINGS' FILE:
#    'BI_app_stop_if_idle_user_settings.ps1'
#
# HOW TO AUTOMATE THIS SCRIPT VIA TASK SCHEDULER
#   Open Task Scheduler and create an appropriately named task with one triggers and one action:
#   Task Scheduler
#   'General' tab
#        'Run only when user is logged in'  [REQUIRED] SEE NOTE #1
#        'Run with highest privileges'      [REQUIRED] SEE NOTE #2
#   'Triggers' tab:
#       'Daily' or 'One time' (e.g., at 12:05 AM, every 15 minutes)
#       (Note: I find 'One time' to be more reliable)
#   'Actions' tab:
#       Program/script: powershell.exe
#       Add arguments: {your_powershell_scripts_path)\{script_file_name}.ps1
#
# NOTES:
#   #1: This setting UNFORTUNATELY results in a flashing PowerShell window when the task runs.
#       While this issue can generally be avoided by running the task with setting:
#           --> General tab > 'Run whether user is logged in or not'
#	    this is not a viable option b/c it causes this script to fail to acquire the desired idle time. 
#       Therefore, we must use the specified setting.
#   #2: 'taskkill' requires administrative privileges to kill a task.
#
# REFS: 
#    https://stackoverflow.com/a/15846912/13499984 ... Win32 snippet used below
#    https://stackoverflow.com/a/50561117/13499984 ... tasklist response
#    https://www.ctrl.blog/entry/idle-task-scheduler-powershell.html
#    https://stackoverflow.com/a/1802836/13499984 ... hide Powershell window when run from Task Scheduler
#    https://superuser.com/a/1500462 ... taskkill /F option leaves icons in system tray

# MUST BE THE FIRST LINE IN A SCRIPT
param (
	[Switch]$Demo,
	[Switch]$NoBiLog,
	[Switch]$NoPushover,
	[Switch]$Help
)

#Set-StrictMode  -Version 3
#Start-Transcript  # default destination is C:\Users\{user}l\Documents

# For debugging
#Write-Host "Demo = $Demo"
#Write-Host "NoBiLog = $NoBiLog"
#Write-Host "NoPushover = $NoPushover"
#Write-Host "Help = $Help"

# INITIALIZE VARS
$ScriptName = Split-Path -leaf $PSCommandpath
$ScriptBaseName = $ScriptName.Replace('.ps1', '')
$PsVersion = $PSVersionTable.PSVersion.Major
# For testing
#Write-Host "ScriptName = $ScriptName"
#Write-Host "PsVersion = $PsVersion"

# NOTE: Scripts running from the following hosts have process names as follows:
#       Task Scheduler is 'svchost', Powershell-ISE is 'explorer', Windows Terminal is 'WindowsTerminal'
#       https://stackoverflow.com/a/65894924
$ParentProcess = (Get-Process -Id (Get-CimInstance Win32_Process -Filter "ProcessID = $pid").ParentProcessId).Name

# For testing
#Write-Host "ParentProcess = $ParentProcess"

# NOTE: Scripts called by another script have a 'PSCommandPath'.
#       Method 'PSCommandPath' returns null if script is executed via the console
#       https://stackoverflow.com/a/23370618/13499984
$CallingScript = $MyInvocation.PSCommandPath

# For testing
#Write-Host "CallingScript = $CallingScript"

# DETERMINE IF RUNNING IN BACKGROUND (IN CLI MODE, OR FROM BLUE IRIS)
$fromConsole = !(($ParentProcess -eq 'svchost' -or $ParentProcess -eq 'BlueIris'))

# @@@ DEV - For testing
#$fromConsole = $false

# For debugging
#Write-Host "fromConsole = $fromConsole"

# Get the current server domain (hostname) and username
# Used for background messaging via msg.exe
# ... tip: from the console, you can type 'hostname' or 'whoami'
$server_name = (whoami).Split('\')[0]
$server_username = (whoami).Split('\')[1]

# For testing
#msg.exe $server_username /TIME:5 "From script... $ScriptName`n-----`nPowerShell version is... $PsVersion`nParent process is... '$ParentProcess'`nCalling script name... $CallingScript"
#msg.exe $server_username /TIME:5 "From script... $ScriptName`nfromConsole = $fromConsole"

# ==============================================================
# IMPORT USER SETTINGS...
# Note: The 'user settings' script is expected to be in same folder as this script
$user_settings_filename = $ScriptName.Replace(".ps1", "_user_settings.ps1")
try {
	. $user_settings_filename
	
	# For debugging
	#Write-Host "Loaded user settings from '$user_settings_filename'"
}
catch {
	$err_msg = "ERR/failed to load user settings file '$user_settings_filename'`nTip: try executing it directly to confirm it exists and reveal syntax errors"
	return $err_msg
}

# VALIDATE USER SETTINGS
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
if (1 -eq 1) {  # Convenience device to hide this lengthy code section while editing

	# Handle REQUIRED user settings
	
	$variable_test = Get-Variable -Name idle_time_threshold_min -Scope Local -ErrorAction SilentlyContinue
	if ($null -eq $variable_test) {
		$idle_time_threshold_min = 30
		Write-Warning "user-settings file is missing setting `$idle_time_threshold_min, using 30 min default"
	} 

	# Handle OPTIONAL user settings
	
	$variable_test = Get-Variable -Name kill_other_processes -Scope Local -ErrorAction SilentlyContinue
	if ($null -eq $variable_test) {
		$kill_other_processes = @()
	}

	$variable_test = Get-Variable -Name post_to_BI_logfile_flg -Scope Local -ErrorAction SilentlyContinue
	if ($null -eq $variable_test) {
		$post_to_BI_logfile_flg = $false
	}
	if ($post_to_BI_logfile_flg) {
		$variable_test = Get-Variable -Name bi_webserver -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$err_msg = "ERR/user-settings file is missing setting `$bi_webserver"
			return $err_msg
		} else {
			if ($bi_webserver.StartsWith('YOUR')) {
				$err_msg = "ERR/user-setting file setting `$bi_webserver is not set"
				return $err_msg
			}
		}
		$variable_test = Get-Variable -Name post_w_creds_flg -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$post_w_creds_flg = $false
		}
		$variable_test = Get-Variable -Name bi_user_name -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$err_msg = "ERR/user-settings file is missing setting `$bi_user_name"
			return $err_msg
		} else {
			if ($bi_user_name.StartsWith('YOUR_')) {
				$err_msg = "ERR/user-settings file setting `$bi_user_name is not set"
				return $err_msg
			}
		}
		$variable_test = Get-Variable -Name bi_user_pw -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$err_msg = "ERR/user-settings file is missing setting `$bi_user_pw"
			return $err_msg
		} else {
			if ($bi_user_pw.StartsWith('YOUR_')) {
				$err_msg = "ERR/user-settings file setting `$bi_user_pw is not set"
				return $err_msg
			}
		}
	}

	$variable_test = Get-Variable -Name send_po_notification_flg -Scope Local -ErrorAction SilentlyContinue
	if ($null -eq $variable_test) {
		$send_po_notification_flg = $false
	}
	if ($send_po_notification_flg) {
		$variable_test = Get-Variable -Name po_app_token -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$err_msg = "ERR/user-settings file is missing setting `$po_app_token"
			return $err_msg
		} else {
			if ($po_app_token.StartsWith('YOUR')) {
				$err_msg = "ERR/user-setting file setting `$po_app_token is not set"
				return $err_msg
			}
		}
		$variable_test = Get-Variable -Name po_user_token -Scope Local -ErrorAction SilentlyContinue
		if ($null -eq $variable_test) {
			$err_msg = "ERR/user-settings file is missing setting `$po_user_token"
			return $err_msg
		} else {
			if ($po_user_token.StartsWith('YOUR')) {
				$err_msg = "ERR/user-setting file setting `$po_user_token' is not set"
				return $err_msg
			}
		}
	}
}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Use Win32 API GetLastInputInfo
# ... Attribution: code source: https://stackoverflow.com/a/15846912/13499984
# NOTE: the code must start in the leftmost column
if (1 -eq 1) {  # Convenience device to hide this lengthy code section while editing
Add-Type @'
using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace PInvoke.Win32 {

    public static class UserInput {

        [DllImport("user32.dll", SetLastError=false)]
        private static extern bool GetLastInputInfo(ref LASTINPUTINFO plii);

        [StructLayout(LayoutKind.Sequential)]
        private struct LASTINPUTINFO {
            public uint cbSize;
            public int dwTime;
        }

        public static DateTime LastInput {
            get {
                DateTime bootTime = DateTime.UtcNow.AddMilliseconds(-Environment.TickCount);
                DateTime lastInput = bootTime.AddMilliseconds(LastInputTicks);
                return lastInput;
            }
        }

        public static TimeSpan IdleTime {
            get {
                return DateTime.UtcNow.Subtract(LastInput);
            }
        }

        public static int LastInputTicks {
            get {
                LASTINPUTINFO lii = new LASTINPUTINFO();
                lii.cbSize = (uint)Marshal.SizeOf(typeof(LASTINPUTINFO));
                GetLastInputInfo(ref lii);
                return lii.dwTime;
            }
        }
    }
}
'@
}
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function CloseBlueIrisPlus() {
# PURPOSE:  Close the Blue Iris application (NOT THE SERVICE!), 
#           and kill plus other processes, if requested
# ARGS:     see param line below
# RETURNS:  nothing; outputs to Console
# REQUIRES: functions: PostCustomBiLogEntry(), SendPushover()

	param (
		[String[]]$KillOtherProcesses,
		[boolean]$DemoFlg,
		[boolean]$FromConsoleFlg
	)

	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#msg.exe $script:server_username /TIME:10 "Entered function $fn_name() ..."
	#Write-Host "Entered function $fn_name() ..."
	#Write-Host ("$fn_name() KillOtherProcesses type = " + $KillOtherProcesses.GetType())
	#Write-Host "$fn_name() KillOtherProcesses = $KillOtherProcesses"
	#Write-Host "$fn_name() DemoFlg = $DemoFlg"
	#Write-Host "$fn_name() FromConsoleFlg = $FromConsoleFlg"
	
	# CLOSE BLUE IRIS APPLICATION
	
	# We can use either of the following... both work when executed from cmd.exe 
	#   Get-Process BlueIris | Foreach-Object { $_.CloseMainWindow() | Out-Null } | stop-process -force
	#   taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris"
	# REF: https://stackoverflow.com/a/58775751/13499984
	
	# NOTE: We try a normal close before resorting to a hard close (/F)
	# KEY POINT: THE HARD CLOSE LEAVES AN ICON IN THE SYSTEM TRAY
	taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris"   # switch /FI filters the service, not the application
	taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris" /F
	
	# ALSO KILL OTHER PROCESSES (IF REQUESTED)
	$msg_addon = ""
	$killed_processes_ary = @()
	if ($KillOtherProcesses.Count -gt 0) {
		foreach ($process_name in $KillOtherProcesses) {
			if (Get-Process "$process_name" -ErrorAction SilentlyContinue) {
				# NOTE: We try a normal close before resorting to a hard close (-Force); 
				# THE LATER LEAVES AN ICON IN SYSTEM TRAY (IF THE SERVICE SHOWS ONE).
				Get-Process "$process_name" -ErrorAction SilentlyContinue | Stop-Process
				Get-Process "$process_name" -ErrorAction SilentlyContinue | Stop-Process -Force
				
				$killed_processes_ary += $process_name 
				
			} else {
				
				# For debugging
				#Write-Host "'$process_name' process not running"
			}
		}
		
		if ($FromConsoleFlg) {
			if ($killed_processes_ary.Count -gt 0) {
				Write-Host ("`nAlso terminated these processes: " + ($killed_processes_ary -Join ', '))
			} else {			
				Write-Host "`nAlso terminated these processes: <none>"
			}
		}
	}
	
	# POST CUSTOM MESSAGE TO BLUE IRIS LOGFILE
	if ($script:post_to_BI_logfile_flg -and !$script:NoBiLog) {

		# Build message
		$str = "Blue%20Iris%20app%20closed%20(" + $script:ScriptName + ")"
		if ($DemoFlg) {
			$msg = "(demo%20mode)%20" + $str
		} else {
			$msg = $str
		}
		$level = 0
		
		# Do it
		$result_str = PostCustomBiLogEntry $msg $level

		if ($FromConsoleFlg) {
			if ($result_str -match 'success') {
				Write-Host ("`nCustom BI logfile message posted...`n" + "'"+ $msg.Replace('%20', ' ') + "'")
			} else {
				Write-Warning "message not posted to Blue Iris logfile"
			}
		}

	}

	# SEND PUSHOVER NOTIFICATION (IF REQUESTED)
	if ($script:send_po_notification_flg -and !$script:NoPushover) {

		# Make message timestamp
		$hhmm = Get-Date -Format HH:mm

		# Make message add-on for other processes closed
		if ($KillOtherProcesses.Count -gt 0) {
			$po_msg_addon = "Add'l processes sought: " + ($KillOtherProcesses -Join ', ')
			if ($killed_processes_ary.Count -gt 0) {
				$po_msg_addon += "`nAdd'l processes closed: " + ($killed_processes_ary -Join ', ')
			} else {
				$po_msg_addon += "`nAdd'l processes closed: <none>"
			}
		} else {
			$po_msg_addon = "Add'l processes sought: <none>"
			$po_msg_addon += "`nAdd'l processes closed: <none>"
		}

		# Finish message
		if ($DemoFlg) {
			$msg = "HEADS UP: Blue Iris app closed @$hhmm`nidle time (" + $script:idle_time_sec + " sec) exceeds threshold (" + $script:idle_time_threshold_sec + ")`n" + $po_msg_addon + "`n(demo mode)"
		} else {
			$msg = "HEADS UP: Blue Iris app closed @$hhmm`nidle time (" + $script:idle_time_min + " min) exceeds threshold (" + $script:idle_time_threshold_min + ")`n" + $po_msg_addon
		}

		# Do it
		$result_str = SendPushover $msg $script:po_priority

		if ($FromConsoleFlg) {
			if ($result_str -match 'success') {
				Write-Host ("`nPushover notification sent...`n" + "'" + $msg + "'")
			} else {
				Write-Warning "Pushover notification not sent"
			}
		}
		
	}

}

function PostCustomBiLogEntry($msg, $level) {
# PURPOSE: Add custom entry to the Blue Iris logfile
# ARGS:    $msg   = (string) NOTE: be sure spaces are replaced with '%20'
#          $level = (integer) 0|1|2 (0:info, 1:warn, 2:error)
#                   NOTE: level=2 can trigger a status alert if the 'Send only if there are new Errors'  
#                   box is checked on the 'Status alerts' dialog... (see the button at bottom of the 
#                   'Blue Iris Status' (log) dialog)
# RETURNS: (string) 'success' or 'fail'

	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#msg.exe $script:server_username /TIME:10 "Entered function $fn_name() ..."
	#Write-Host "Entered function $fn_name() ..."

	$rtn_str = 'fail'

	if ($script:post_w_creds_flg) {
		$uri = "http://$bi_webserver/admin?log=$msg&level=$level&user=$script:bi_user_name&pw=$script:bi_user_pw"
	} else {
		$uri = "http://$bi_webserver/admin?log=$msg&level=$level"
	}
	
	# For debugging
	#Write-Host "$fn_name() uri = $uri"
	
	try {
		
		#$web_response_obj = Invoke-WebRequest -Uri $uri -UseBasicParsing
		$web_response_obj = Invoke-WebRequest -Uri $uri
		
		$rtn_str = 'success'
	
		# For debugging
		#Write-Host ("$fn_name() web_response_obj = " + ($web_response_obj | ConvertTo-Json -Depth 32))
		
	} 
	catch {

		# For debugging
		#Write-Host "$error
		#Write-Host $_.ScriptStackTrace

		if ($script:fromConsole) {
			Write-Host ("$fn_name() " + $_.Exception.Message) -ForegroundColor Red
		}

		$rtn_str = 'fail'

	}
	
	# For debugging
	#Write-Host "$fn_name() rtn_str = $rtn_str"
	
	return $rtn_str

}

function SendPushover($msg_str, $priority_level) {
# PURPOSE: Send custom notification via the Pushover API
# ARGS:    $msg            = (string)
#          $priority_level = (integer) -2|-1|0|1|2; -2=lowest, -1=low(silent), 0=normal, 1=high-priority, 2=emergency
# RETURNS: (string) 'success' or 'fail'

	$fn_name = (Get-Variable MyInvocation).Value.MyCommand.Name

	# For debugging
	#msg.exe $script:server_username /TIME:10 "Entered function $fn_name() ..."
	#Write-Host "Entered function $fn_name() ..."
	#Write-Host "$fn_name() priority_level = $priority_level"

	$rtn_str = 'fail'

	$uri = 'https://api.pushover.net/1/messages.json'
	$parameters = @{
		token    = $script:po_app_token
		user     = $script:po_user_token
		priority = $priority_level
		retry    = $script:po_retry
		expire   = $script:po_expire
		sound    = $script:po_sound
		device   = $script:po_device
		message  = $msg_str
	}
	#$parameters | Invoke-RestMethod -Uri $uri -Method Post
	$response = $parameters | Invoke-RestMethod -Uri $uri -Method Post
	
	if ($response) {
		if ($response.status -eq 1) {
			$rtn_str = 'success'
		}
	}
	
	# For debugging
	#Write-Host ("$fn_name() parameters = " + ($parameters | ConvertTo-Json))
	#Write-Host ("$fn_name() response = " + ($response | ConvertTo-Json))
	#Write-Host "$fn_name() rtn_str = $rtn_str"

	return $rtn_str

}

if ($Help) {
	if ($fromConsole) {
		Write-Host "===================================================================================="
		Write-Host "Blue Iris Utility - Close Application (not service) if PC is Idle"
		Write-Host "------------------------------------------------------------------------------------"
		Write-Host "Usage: $ScriptName {no args} ... kill BI per Defaults (see below)"
		Write-Host "       $ScriptName -Demo ... simulate app's operation with messaging"
		Write-Host "       $ScriptName -Demo -NoBiLog -NoPushover ... simulate w/o messaging"
		Write-Host "Args:"
		Write-Host "   -Demo ... use to simulate how the script works (w/ 10 sec idle threshold)"
		Write-Host "   -NoBiLog ... temporarily disable event logging to the Blue Iris logfile"
		Write-Host "   -NoPushover ... temporarily disable notifications via the Pushover API"
		Write-Host "   -Help ... show this usage guide"
		Write-Host "Defaults: (per the user_settings file)"
		Write-Host "   `$idle_time_threshold_min = $idle_time_threshold_min min"
		Write-Host "   `$post_to_BI_logfile_flg = $post_to_BI_logfile_flg"
		Write-Host "   `$send_po_notification_flg = $send_po_notification_flg"
		Write-Host "------------------------------------------------------------------------------------"
		Write-Host ""
	}
	exit
}

# TEST FOR BLUE IRIS APPLICATION
# 'Tasklist' method... Ref: https://stackoverflow.com/a/50561117/13499984
# Note: If the Blue Iris desktop application is NOT running, then 
#       $response = "INFO: No tasks are running which match the specified criteria."
#       If the app is running, then the response is like...
#       $response = "Image Name,PID,Session Name,Session#,Mem Usage
#                  BlueIris.exe,18152,RDP-Tcp#27,2,99,467 K"
$response = tasklist /FI "IMAGENAME eq BlueIris.exe" /FI "SESSIONNAME ne Services" /FO CSV

# For debugging
#Write-Host "Response from 'tasklist' command... "
#Write-Host ("'" + $response + "'") -ForegroundColor Cyan
#msg.exe $server_username /TIME:10 "response from 'tasklist' command... `n'$response'"

$target_str = 'INFO: No tasks are running which match the specified criteria.'
if ($response -eq $target_str) {
	
	$rtn_str = "Nothing to do: the Blue Iris desktop application (blueiris.exe) is not running"

	# For debugging
	#msg.exe $server_username /TIME:10 "$rtn_str"

	return $rtn_str

} elseif ($response -match 'BlueIris.exe') {
	
	if ($Demo) {
		
		# DEMO MODE
		
		Write-Host ("Demo mode...")		
		
		$idle_time_threshold_sec = 10
		for ($i = 0; $i -lt 10; $i++) {
			
			# Get Blue Iris PC's idle time
			$last_input_datetime = [PInvoke.Win32.UserInput]::LastInput
			$idle_time_str = [PInvoke.Win32.UserInput]::IdleTime
			$timespan = [TimeSpan]::Parse($idle_time_str)
			$idle_time_sec = [math]::round($timespan.TotalSeconds,1)

			Write-Host ("Last input: " + $last_input_datetime.ToLocalTime())
			Write-Host ("Idle for: " + $idle_time_str + " (" + $idle_time_sec + " sec)")
		
			if ($idle_time_sec -gt $idle_time_threshold_sec) {

				Write-Host ("idle time exceeds threshold (" + $idle_time_threshold_sec + " sec)")
				
				if ($kill_other_processes.Count -gt 0) {
					Write-Host ("`nAttempting to close the Blue Iris application and other processes...")
				} else {
					Write-Host ("`nAttempting to close the Blue Iris application...")
				}
				
				CloseBlueIrisPlus `
				-KillOtherProcesses $kill_other_processes `
				-DemoFlg $true `
				-FromConsoleFlg $fromConsole
				
				break
			
			}

			Start-Sleep -Seconds (Get-Random -Minimum 2 -Maximum 5)
		}
		
	} else {
		
		# NORMAL MODE
		
		# Get Blue Iris PC's idle time
		$last_input_datetime = [PInvoke.Win32.UserInput]::LastInput
		$idle_time_str = [PInvoke.Win32.UserInput]::IdleTime
		$timespan = [TimeSpan]::Parse($idle_time_str)
		$idle_time_min = [math]::round($timespan.TotalMinutes,3)
		
		# @@@ Dev - For testings
		#$idle_time_min = 35

		if ($fromConsole) {
			Write-Host ("Last input: " + $last_input_datetime.ToLocalTime())
			Write-Host ("Idle for: " + $idle_time_str + " (" + $idle_time_min + " min)")
		}

		# For debugging.. HANDY WHEN TESTING FROM TASK SCHEDULER
		#$msg = "BI PC...`nlast input = " + $last_input_datetime.ToLocalTime() + "`nidle time = $idle_time_str`nidle_time_min = $idle_time_min"
		#msg.exe $server_username /TIME:10 $msg

		# Test for idle time trigger
		if ($idle_time_min -gt $idle_time_threshold_min) {

			CloseBlueIrisPlus `
			-KillOtherProcesses $kill_other_processes `
			-DemoFlg $false `
			-FromConsoleFlg $fromConsole
			
		} else {
			$rtn_str = "Nothing to do: PC idle time ($idle_time_min min) < threshold ($idle_time_threshold_min min)"
			return $rtn_str
		}
	}
	
} else {

	$rtn_str = "Nothing to do: unexpected response from 'tasklist' command"
	
	# For testing/debugging
	msg.exe $server_username /TIME:10 $rtn_str	
	
	return $rtn_str
}

if ($fromConsole) {
	Write-Host ""
}

#Stop-Transcript